import React, { useState, useEffect, useRef } from 'react';
import { Modal, ModalBody, ModalFooter, Button, Row, Col, ButtonGroup, Breadcrumb, BreadcrumbItem, Alert, UncontrolledDropdown, DropdownToggle, DropdownMenu, DropdownItem, Spinner, ModalHeader, Table, Progress } from 'reactstrap';
import { faPaperPlane, faTimes, faFile, faFileExcel, faFileWord, faFileImage, faFileVideo, faFileAudio, faFilePdf, faFilePowerpoint, faFileArchive, faPlus, faEdit, faTrash, faFolder, faCog, faAngleDown, faDownload, faAngleRight, faFolderOpen, faUpload, faCheck } from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { FormikWrapper, InputField } from '@neolution-ch/react-formik-ui';
import { jsx, jsxs } from 'react/jsx-runtime';
import Dropzone from 'react-dropzone';

function createCommonjsModule(fn) {
  var module = { exports: {} };
	return fn(module, module.exports), module.exports;
}

var rngBrowser = createCommonjsModule(function (module) {
// Unique ID creation requires a high quality random # generator.  In the
// browser this is a little complicated due to unknown quality of Math.random()
// and inconsistent support for the `crypto` API.  We do the best we can via
// feature-detection

// getRandomValues needs to be invoked in a context where "this" is a Crypto
// implementation. Also, find the complete implementation of crypto on IE11.
var getRandomValues = (typeof(crypto) != 'undefined' && crypto.getRandomValues && crypto.getRandomValues.bind(crypto)) ||
                      (typeof(msCrypto) != 'undefined' && typeof window.msCrypto.getRandomValues == 'function' && msCrypto.getRandomValues.bind(msCrypto));

if (getRandomValues) {
  // WHATWG crypto RNG - http://wiki.whatwg.org/wiki/Crypto
  var rnds8 = new Uint8Array(16); // eslint-disable-line no-undef

  module.exports = function whatwgRNG() {
    getRandomValues(rnds8);
    return rnds8;
  };
} else {
  // Math.random()-based (RNG)
  //
  // If all else fails, use Math.random().  It's fast, but is of unspecified
  // quality.
  var rnds = new Array(16);

  module.exports = function mathRNG() {
    for (var i = 0, r; i < 16; i++) {
      if ((i & 0x03) === 0) r = Math.random() * 0x100000000;
      rnds[i] = r >>> ((i & 0x03) << 3) & 0xff;
    }

    return rnds;
  };
}
});

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */
var byteToHex = [];
for (var i = 0; i < 256; ++i) {
  byteToHex[i] = (i + 0x100).toString(16).substr(1);
}

function bytesToUuid(buf, offset) {
  var i = offset || 0;
  var bth = byteToHex;
  // join used to fix memory issue caused by concatenation: https://bugs.chromium.org/p/v8/issues/detail?id=3175#c4
  return ([
    bth[buf[i++]], bth[buf[i++]],
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]],
    bth[buf[i++]], bth[buf[i++]],
    bth[buf[i++]], bth[buf[i++]]
  ]).join('');
}

var bytesToUuid_1 = bytesToUuid;

// **`v1()` - Generate time-based UUID**
//
// Inspired by https://github.com/LiosK/UUID.js
// and http://docs.python.org/library/uuid.html

var _nodeId;
var _clockseq;

// Previous uuid creation time
var _lastMSecs = 0;
var _lastNSecs = 0;

// See https://github.com/uuidjs/uuid for API details
function v1(options, buf, offset) {
  var i = buf && offset || 0;
  var b = buf || [];

  options = options || {};
  var node = options.node || _nodeId;
  var clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq;

  // node and clockseq need to be initialized to random values if they're not
  // specified.  We do this lazily to minimize issues related to insufficient
  // system entropy.  See #189
  if (node == null || clockseq == null) {
    var seedBytes = rngBrowser();
    if (node == null) {
      // Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
      node = _nodeId = [
        seedBytes[0] | 0x01,
        seedBytes[1], seedBytes[2], seedBytes[3], seedBytes[4], seedBytes[5]
      ];
    }
    if (clockseq == null) {
      // Per 4.2.2, randomize (14 bit) clockseq
      clockseq = _clockseq = (seedBytes[6] << 8 | seedBytes[7]) & 0x3fff;
    }
  }

  // UUID timestamps are 100 nano-second units since the Gregorian epoch,
  // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
  // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
  // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.
  var msecs = options.msecs !== undefined ? options.msecs : new Date().getTime();

  // Per 4.2.1.2, use count of uuid's generated during the current clock
  // cycle to simulate higher resolution clock
  var nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1;

  // Time since last uuid creation (in msecs)
  var dt = (msecs - _lastMSecs) + (nsecs - _lastNSecs)/10000;

  // Per 4.2.1.2, Bump clockseq on clock regression
  if (dt < 0 && options.clockseq === undefined) {
    clockseq = clockseq + 1 & 0x3fff;
  }

  // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
  // time interval
  if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
    nsecs = 0;
  }

  // Per 4.2.1.2 Throw error if too many uuids are requested
  if (nsecs >= 10000) {
    throw new Error('uuid.v1(): Can\'t create more than 10M uuids/sec');
  }

  _lastMSecs = msecs;
  _lastNSecs = nsecs;
  _clockseq = clockseq;

  // Per 4.1.4 - Convert from unix epoch to Gregorian epoch
  msecs += 12219292800000;

  // `time_low`
  var tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
  b[i++] = tl >>> 24 & 0xff;
  b[i++] = tl >>> 16 & 0xff;
  b[i++] = tl >>> 8 & 0xff;
  b[i++] = tl & 0xff;

  // `time_mid`
  var tmh = (msecs / 0x100000000 * 10000) & 0xfffffff;
  b[i++] = tmh >>> 8 & 0xff;
  b[i++] = tmh & 0xff;

  // `time_high_and_version`
  b[i++] = tmh >>> 24 & 0xf | 0x10; // include version
  b[i++] = tmh >>> 16 & 0xff;

  // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)
  b[i++] = clockseq >>> 8 | 0x80;

  // `clock_seq_low`
  b[i++] = clockseq & 0xff;

  // `node`
  for (var n = 0; n < 6; ++n) {
    b[i + n] = node[n];
  }

  return buf ? buf : bytesToUuid_1(b);
}

var v1_1 = v1;

function v4(options, buf, offset) {
  var i = buf && offset || 0;

  if (typeof(options) == 'string') {
    buf = options === 'binary' ? new Array(16) : null;
    options = null;
  }
  options = options || {};

  var rnds = options.random || (options.rng || rngBrowser)();

  // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
  rnds[6] = (rnds[6] & 0x0f) | 0x40;
  rnds[8] = (rnds[8] & 0x3f) | 0x80;

  // Copy bytes to buffer, if provided
  if (buf) {
    for (var ii = 0; ii < 16; ++ii) {
      buf[i + ii] = rnds[ii];
    }
  }

  return buf || bytesToUuid_1(rnds);
}

var v4_1 = v4;

var uuid = v4_1;
uuid.v1 = v1_1;
uuid.v4 = v4_1;

var uuid_1 = uuid;

var defaultTranslations = {
  en: {
    errorRequired: "Input is mandatory",
    textDeleteConfirmFile: "Are you sure you want to permanently delete this file?",
    textdeleteConfirmFolder: "Are you sure you want to delete this folder? All subfolders (including their files) will also be deleted.",
    textDropZone: "Drag n drop some files here, or click to select files",
    labelName: "Name",
    labelDateModified: "Date Modified",
    labelSize: "File Size",
    labelProgress: "Progress",
    labelYes: "Yes",
    labelCancel: "Cancel",
    labelSubmit: "Send",
    labelAddFolder: "Add folder",
    labelDelete: "Delete",
    labelDownload: "Download",
    warningSizeExceeded: "The file size limit has exceeded."
  },
  it: {
    errorRequired: "L'inserimento è obbligatorio",
    textDeleteConfirmFile: "Sei sicuro di voler eliminare definitivamente questo file?",
    textdeleteConfirmFolder: "Sei sicuro di voler cancellare questa cartella? Anche tutte le sottocartelle (compresi i loro file) saranno cancellate.",
    textDropZone: "Trascina qui alcuni file, o clicca per selezionare i file",
    labelName: "Nome",
    labelDateModified: "Data modificata",
    labelSize: "Dimensione del file",
    labelProgress: "Progresso",
    labelYes: "Si",
    labelCancel: "Cancella",
    labelSubmit: "Invia",
    labelAddFolder: "aggiungi nuovo",
    labelDelete: "cancellare",
    labelDownload: "scaricare",
    warningSizeExceeded: "Il limite di dimensione del file è stato superato."
  },
  de: {
    errorRequired: "Angabe zwingend",
    textDeleteConfirmFile: "Soll der Eintrag endgültig gelöscht werden?",
    textdeleteConfirmFolder: "Soll dieser Ordner endgültig gelöscht werden? Alle Unterordner und alle zugehörigen Dateien werden gelöscht.",
    textDropZone: "Dateien per Drag & Drop hierher ziehen oder klicken, um Dateien auszuwählen",
    labelName: "Name",
    labelDateModified: "Änderungsdatum",
    labelSize: "Dateigrösse",
    labelProgress: "Fortschritt",
    labelYes: "Ja",
    labelCancel: "Abbruch",
    labelSubmit: "Absenden",
    labelAddFolder: "Ordner hinzufügen",
    labelDelete: "Löschen",
    labelDownload: "Download",
    warningSizeExceeded: "Die Dateigrössenbeschränkung wurde überschritten."
  },
  fr: {
    errorRequired: "Obligatoire",
    textDeleteConfirmFile: "Êtes-vous sûr de vouloir supprimer définitivement ce fichier?",
    textdeleteConfirmFolder: "Êtes-vous sûr de vouloir supprimer ce dossier? Tous les sous-dossiers (et les fichiers qu'ils contiennent) seront également supprimés.",
    textDropZone: "Faites glisser et déposez les fichiers ici ou cliquez pour sélectionner des fichiers",
    labelName: "Nom",
    labelDateModified: "Date modifiée",
    labelSize: "Taille du fichier",
    labelProgress: "Progrès",
    labelYes: "Oui",
    labelCancel: "Annuler",
    labelSubmit: "Soumettre",
    labelAddFolder: "Ajouter le dossier",
    labelDelete: "Supprimer",
    labelDownload: "Télécharger",
    warningSizeExceeded: "La limite de taille de fichier a été dépassée."
  }
};

var Translations = /*#__PURE__*/function () {
  function Translations() {}

  Translations.getTranslations = function getTranslations() {
    return Translations.translations;
  };

  Translations.setTranslations = function setTranslations(lang) {
    Translations.translations = defaultTranslations[lang];
  };

  Translations.setCustomTranslations = function setCustomTranslations(translations) {
    Translations.translations = translations;
  };

  return Translations;
}();

Translations.translations = defaultTranslations.en;

var AddFolderModal = function AddFolderModal(_ref) {
  var _onSubmit = _ref.onSubmit,
      isOpen = _ref.isOpen,
      onClose = _ref.onClose,
      _ref$initialValues = _ref.initialValues,
      initialValues = _ref$initialValues === void 0 ? {
    name: ""
  } : _ref$initialValues;

  var _Translations$getTran = Translations.getTranslations(),
      labelName = _Translations$getTran.labelName,
      errorRequired = _Translations$getTran.errorRequired,
      labelSubmit = _Translations$getTran.labelSubmit,
      labelCancel = _Translations$getTran.labelCancel;

  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsx(Modal, {
      isOpen: isOpen,
      toggle: function toggle() {
        return onClose();
      },
      children: /*#__PURE__*/jsxs(FormikWrapper, {
        initialValues: initialValues,
        validate: function validate(values) {
          var errors = {};
          if (!values.name) errors.name = errorRequired;
          return errors;
        },
        onSubmit: function onSubmit(values) {
          return _onSubmit(values);
        },
        children: [/*#__PURE__*/jsx(ModalBody, {
          children: /*#__PURE__*/jsx(InputField, {
            name: "name",
            label: labelName,
            placeholder: labelName
          })
        }), /*#__PURE__*/jsxs(ModalFooter, {
          children: [/*#__PURE__*/jsxs(Button, {
            type: "submit",
            color: "primary",
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faPaperPlane
            }), " ", labelSubmit]
          }), /*#__PURE__*/jsxs(Button, {
            type: "button",
            color: "danger",
            onClick: function onClick() {
              return onClose();
            },
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faTimes
            }), " ", labelCancel]
          })]
        })]
      })
    })
  });
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function formatFileSize(bytes, si, dp) {
  if (si === void 0) {
    si = true;
  }

  if (dp === void 0) {
    dp = 1;
  }

  var thresh = si ? 1000 : 1024;

  if (Math.abs(bytes) < thresh) {
    return bytes + " B";
  }

  var units = si ? ["KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"] : ["KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"];
  var u = -1;
  var r = Math.pow(10, dp);

  do {
    // eslint-disable-next-line no-param-reassign
    bytes /= thresh;
    u += 1;
  } while (Math.round(Math.abs(bytes) * r) / r >= thresh && u < units.length - 1);

  return bytes.toFixed(dp) + " " + units[u];
}

function formatDatetime(date, format) {
  if (format === void 0) {
    format = "dd.MM.yyyy HH:mm:ss";
  }

  var padStart = function padStart(value) {
    return value.toString().padStart(2, "0");
  };

  var typedDate = typeof date === "string" ? new Date(date) : date;
  return format.replace(/yyyy/g, padStart(typedDate.getFullYear())).replace(/dd/g, padStart(typedDate.getDate())).replace(/MM/g, padStart(typedDate.getMonth() + 1)).replace(/HH/g, padStart(typedDate.getHours())).replace(/mm/g, padStart(typedDate.getMinutes())).replace(/ss/g, padStart(typedDate.getSeconds()));
}

var fileTypeLut = {
  ".xls": faFileExcel,
  ".xlsx": faFileExcel,
  ".doc": faFileWord,
  ".docx": faFileWord,
  ".png": faFileImage,
  ".jpeg": faFileImage,
  ".mov": faFileVideo,
  ".mp4": faFileVideo,
  ".mp3": faFileAudio,
  ".pdf": faFilePdf,
  ".ppt": faFilePowerpoint,
  ".pptx": faFilePowerpoint,
  ".rar": faFileArchive,
  ".zip": faFileArchive
};

var FileIconFormatter = function FileIconFormatter(_ref) {
  var _fileTypeLut$fileType;

  var fileType = _ref.fileType;
  return /*#__PURE__*/jsx(FontAwesomeIcon, {
    className: "mr-3",
    size: "3x",
    icon: (_fileTypeLut$fileType = fileTypeLut[fileType]) != null ? _fileTypeLut$fileType : faFile
  });
};

var FolderContent = function FolderContent(_ref) {
  var _files$length;

  var folders = _ref.folders,
      selectedFolderId = _ref.selectedFolderId,
      onRequestAddFolder = _ref.onRequestAddFolder,
      onRequestDeleteFolder = _ref.onRequestDeleteFolder,
      onRequestEditFolder = _ref.onRequestEditFolder,
      onRequestUploadFiles = _ref.onRequestUploadFiles,
      onSelectFolder = _ref.onSelectFolder,
      onRequestDeleteFile = _ref.onRequestDeleteFile,
      files = _ref.files,
      _ref$sizeExceeded = _ref.sizeExceeded,
      sizeExceeded = _ref$sizeExceeded === void 0 ? true : _ref$sizeExceeded;

  var _Translations$getTran = Translations.getTranslations(),
      textDropZone = _Translations$getTran.textDropZone,
      labelDateModified = _Translations$getTran.labelDateModified,
      labelName = _Translations$getTran.labelName,
      warningSizeExceeded = _Translations$getTran.warningSizeExceeded;

  var _useState = useState([]),
      breadCrumbs = _useState[0],
      setBreadCrumbs = _useState[1];

  var selectedFolder = folders.find(function (folder) {
    return folder.id === selectedFolderId;
  });

  var getBreadCrumbs = function getBreadCrumbs() {
    if (!selectedFolder) return [];
    var result = [];
    var start = selectedFolder;

    while (start) {
      result.push(start);
      start = folders.find(function (folder) {
        var _start;

        return folder.id === ((_start = start) == null ? void 0 : _start.parentId);
      });
    }

    return result.reverse();
  };

  useEffect(function () {
    setBreadCrumbs(getBreadCrumbs());
  }, [selectedFolderId]);
  var hasFolders = folders.some(function (x) {
    return x.parentId === selectedFolderId;
  });
  var hasFiles = ((_files$length = files == null ? void 0 : files.length) != null ? _files$length : 0) > 0;
  return /*#__PURE__*/jsx(React.Fragment, {
    children: selectedFolder && /*#__PURE__*/jsxs(React.Fragment, {
      children: [/*#__PURE__*/jsxs(Row, {
        children: [/*#__PURE__*/jsx(Col, {
          lg: 5,
          children: /*#__PURE__*/jsx("h2", {
            style: {
              display: "inline-block"
            },
            children: selectedFolder.name
          })
        }), /*#__PURE__*/jsx(Col, {
          className: "ml-auto",
          children: /*#__PURE__*/jsx("div", {
            className: "text-right",
            children: /*#__PURE__*/jsxs(ButtonGroup, {
              children: [/*#__PURE__*/jsx(Button, {
                color: "primary",
                onClick: function onClick() {
                  onRequestAddFolder(selectedFolder);
                },
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faPlus
                })
              }), /*#__PURE__*/jsx(Button, {
                color: "primary",
                onClick: function onClick() {
                  onRequestEditFolder(selectedFolder);
                },
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faEdit
                })
              }), /*#__PURE__*/jsx(Button, {
                color: "primary",
                onClick: function onClick() {
                  onRequestDeleteFolder(selectedFolder);
                },
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faTrash
                })
              })]
            })
          })
        })]
      }), /*#__PURE__*/jsx(Row, {
        children: /*#__PURE__*/jsx(Col, {
          xxl: 12,
          children: /*#__PURE__*/jsx(Breadcrumb, {
            children: breadCrumbs.length >= 2 && /*#__PURE__*/jsx(React.Fragment, {
              children: breadCrumbs.map(function (x, i) {
                return /*#__PURE__*/jsx(BreadcrumbItem, {
                  active: i === breadCrumbs.length - 1,
                  children: /*#__PURE__*/jsx("a", {
                    onClick: function onClick() {
                      return onSelectFolder(x.id);
                    },
                    children: x.name
                  })
                }, x.id);
              })
            })
          })
        })
      }), sizeExceeded ? /*#__PURE__*/jsx(Alert, {
        color: "danger",
        children: warningSizeExceeded
      }) : /*#__PURE__*/jsx(Dropzone, {
        onDrop: function onDrop(acceptedFiles) {
          try {
            onRequestUploadFiles(selectedFolder.id, acceptedFiles);
            return Promise.resolve();
          } catch (e) {
            return Promise.reject(e);
          }
        },
        children: function children(_ref2) {
          var getRootProps = _ref2.getRootProps,
              getInputProps = _ref2.getInputProps,
              isDragActive = _ref2.isDragActive;
          return /*#__PURE__*/jsx(Col, {
            lg: 12,
            style: {
              padding: "20px",
              borderWidth: 2,
              borderRadius: 2,
              borderColor: isDragActive ? "blue" : "#eeeeee",
              borderStyle: "dashed",
              backgroundColor: "#fafafa",
              color: "#bdbdbd",
              outline: "none",
              transition: "border .24s ease-in-out"
            },
            children: /*#__PURE__*/jsxs("div", _extends({}, getRootProps(), {
              children: [/*#__PURE__*/jsx("input", _extends({}, getInputProps())), /*#__PURE__*/jsx("p", {
                children: textDropZone
              })]
            }))
          });
        }
      }), (hasFolders || hasFiles) && /*#__PURE__*/jsxs("table", {
        className: "mb-0 table table-hover",
        children: [/*#__PURE__*/jsx("thead", {
          children: /*#__PURE__*/jsxs("tr", {
            children: [/*#__PURE__*/jsx("th", {
              className: "align-middle bt-0",
              children: labelName
            }), /*#__PURE__*/jsx("th", {
              className: "align-middle bt-0",
              children: labelDateModified
            }), /*#__PURE__*/jsx("th", {
              className: "align-middle bt-0 text-right",
              children: "\xA0"
            })]
          })
        }), /*#__PURE__*/jsxs("tbody", {
          children: [folders.filter(function (x) {
            return x.parentId === selectedFolderId;
          }).map(function (x) {
            return /*#__PURE__*/jsxs("tr", {
              children: [/*#__PURE__*/jsx("td", {
                className: "align-middle",
                children: /*#__PURE__*/jsxs("div", {
                  className: "media",
                  children: [/*#__PURE__*/jsx("div", {
                    className: "media-left media-middle",
                    children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faFolder,
                      size: "3x",
                      className: "mr-3"
                    })
                  }), /*#__PURE__*/jsx("div", {
                    className: "media-body",
                    children: /*#__PURE__*/jsx("div", {
                      className: "text-inverse",
                      children: /*#__PURE__*/jsx("a", {
                        onClick: function onClick() {
                          return onSelectFolder(x.id);
                        },
                        children: x.name
                      })
                    })
                  })]
                })
              }), /*#__PURE__*/jsx("td", {
                className: "align-middle"
              }), /*#__PURE__*/jsx("td", {
                className: "align-middle text-right"
              })]
            }, x.id);
          }), files == null ? void 0 : files.map(function (x) {
            return /*#__PURE__*/jsxs("tr", {
              children: [/*#__PURE__*/jsx("td", {
                className: "align-middle",
                children: /*#__PURE__*/jsxs("div", {
                  className: "media",
                  children: [/*#__PURE__*/jsx("div", {
                    className: "media-left media-middle",
                    children: /*#__PURE__*/jsx(FileIconFormatter, {
                      fileType: x.fileType
                    })
                  }), /*#__PURE__*/jsxs("div", {
                    className: "media-body",
                    children: [/*#__PURE__*/jsx("div", {
                      className: "text-inverse",
                      children: /*#__PURE__*/jsx("a", {
                        target: "_blank",
                        rel: "noreferrer",
                        href: x.url,
                        children: x.name
                      })
                    }), /*#__PURE__*/jsx("span", {
                      children: formatFileSize(x.fileSize)
                    })]
                  })]
                })
              }), /*#__PURE__*/jsx("td", {
                className: "align-middle",
                children: formatDatetime(x.dateModified)
              }), /*#__PURE__*/jsx("td", {
                className: "align-middle text-right",
                children: /*#__PURE__*/jsxs(UncontrolledDropdown, {
                  children: [/*#__PURE__*/jsxs(DropdownToggle, {
                    tag: "button",
                    className: "btn btn-link",
                    children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faCog,
                      className: "mr-2"
                    }), /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faAngleDown
                    })]
                  }), /*#__PURE__*/jsxs(DropdownMenu, {
                    right: true,
                    children: [/*#__PURE__*/jsx("a", {
                      target: "_blank",
                      rel: "noreferrer",
                      href: x.url,
                      children: /*#__PURE__*/jsxs(DropdownItem, {
                        children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
                          icon: faDownload,
                          className: "mr-2"
                        }), "Download"]
                      })
                    }), x.isDeleteable && /*#__PURE__*/jsxs(DropdownItem, {
                      onClick: function onClick() {
                        return onRequestDeleteFile(x.id);
                      },
                      children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
                        icon: faTrash,
                        className: "mr-2"
                      }), "Delete"]
                    })]
                  })]
                })
              })]
            }, x.id);
          })]
        })]
      })]
    })
  });
};

var FolderTree = function FolderTree(props) {
  var _Translations$getTran = Translations.getTranslations(),
      labelAddFolder = _Translations$getTran.labelAddFolder;

  if (props.isLoading) {
    return /*#__PURE__*/jsxs(React.Fragment, {
      children: ["loading...", /*#__PURE__*/jsx("div", {
        className: "spinner-border",
        role: "status",
        children: /*#__PURE__*/jsx("span", {
          className: "sr-only",
          children: "Loading..."
        })
      }), /*#__PURE__*/jsx(Spinner, {
        color: "primary",
        type: "grow",
        children: "Loading..."
      })]
    });
  }

  return /*#__PURE__*/jsxs(React.Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: "tree",
      children: props.folders && /*#__PURE__*/jsx(FolderSubtree, _extends({}, props, {
        folders: props.folders,
        parentIdToDisplay: undefined
      }))
    }), /*#__PURE__*/jsxs("a", {
      onClick: function onClick() {
        return props.onRequestAddFolder(null);
      },
      children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
        icon: faPlus
      }), " ", labelAddFolder]
    })]
  });
};

var FolderSubtree = function FolderSubtree(props) {
  var folders = props.folders,
      onToggleFolder = props.onToggleFolder,
      onSelectFolder = props.onSelectFolder,
      onRequestUploadFiles = props.onRequestUploadFiles,
      onRequestAddFolder = props.onRequestAddFolder,
      onRequestEditFolder = props.onRequestEditFolder,
      onRequestDeleteFolder = props.onRequestDeleteFolder,
      selectedFolderId = props.selectedFolderId,
      parentIdToDisplay = props.parentIdToDisplay;
  var foldersToDisplay = folders == null ? void 0 : folders.filter(function (x) {
    return x.parentId == parentIdToDisplay;
  });
  return /*#__PURE__*/jsx("ul", {
    children: foldersToDisplay == null ? void 0 : foldersToDisplay.map(function (x) {
      return /*#__PURE__*/jsx(React.Fragment, {
        children: /*#__PURE__*/jsxs("li", {
          children: [/*#__PURE__*/jsx("span", {
            style: {
              display: "inline-block",
              width: "10px",
              minWidth: "15px"
            },
            children: folders.some(function (f) {
              return f.parentId == x.id;
            }) && /*#__PURE__*/jsx("a", {
              onClick: function onClick() {
                return onToggleFolder(x.id);
              },
              children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                icon: x.isOpen ? faAngleDown : faAngleRight
              })
            })
          }), /*#__PURE__*/jsx(Dropzone, {
            noClick: true,
            onDrop: function onDrop(acceptedFiles) {
              try {
                return Promise.resolve(onRequestUploadFiles(x.id, acceptedFiles)).then(function () {});
              } catch (e) {
                return Promise.reject(e);
              }
            },
            children: function children(_ref) {
              var getRootProps = _ref.getRootProps,
                  getInputProps = _ref.getInputProps,
                  open = _ref.open;
              return /*#__PURE__*/jsxs("span", {
                className: "actions-trigger",
                children: [/*#__PURE__*/jsx("a", {
                  onClick: function onClick() {
                    onSelectFolder(x.id);
                  },
                  children: /*#__PURE__*/jsxs("span", _extends({}, getRootProps(), {
                    children: [/*#__PURE__*/jsx("input", _extends({}, getInputProps())), /*#__PURE__*/jsxs("span", {
                      style: {
                        fontWeight: x.id === selectedFolderId ? "bold" : "normal"
                      },
                      children: [/*#__PURE__*/jsx("span", {
                        style: {
                          minWidth: "20px",
                          display: "inline-block"
                        },
                        children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                          icon: x.isOpen ? faFolderOpen : faFolder
                        })
                      }), x.name]
                    })]
                  }))
                }), /*#__PURE__*/jsxs("span", {
                  className: "actions ml-2",
                  children: [/*#__PURE__*/jsx("a", {
                    className: "pl-1",
                    onClick: open,
                    children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faUpload
                    })
                  }), /*#__PURE__*/jsx("a", {
                    className: "pl-1",
                    onClick: function onClick() {
                      return onRequestAddFolder(x);
                    },
                    children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faPlus
                    })
                  }), /*#__PURE__*/jsx("a", {
                    className: "pl-1",
                    onClick: function onClick() {
                      return onRequestEditFolder(x);
                    },
                    children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faEdit
                    })
                  }), /*#__PURE__*/jsx("a", {
                    className: "pl-1",
                    onClick: function onClick() {
                      return onRequestDeleteFolder(x);
                    },
                    children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faTrash
                    })
                  })]
                })]
              });
            }
          }), !!x.isOpen && /*#__PURE__*/jsx(FolderSubtree, _extends({}, props, {
            parentIdToDisplay: x.id
          }))]
        })
      }, x.id);
    })
  });
};

var DeleteConfirmModal = function DeleteConfirmModal(_ref) {
  var _onSubmit = _ref.onSubmit,
      isOpen = _ref.isOpen,
      onClose = _ref.onClose,
      isFolder = _ref.isFolder;

  var _Translations$getTran = Translations.getTranslations(),
      textDeleteConfirmFile = _Translations$getTran.textDeleteConfirmFile,
      textdeleteConfirmFolder = _Translations$getTran.textdeleteConfirmFolder,
      labelYes = _Translations$getTran.labelYes,
      labelCancel = _Translations$getTran.labelCancel;

  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsx(Modal, {
      isOpen: isOpen,
      toggle: function toggle() {
        return onClose();
      },
      children: /*#__PURE__*/jsxs(FormikWrapper, {
        initialValues: {},
        onSubmit: function onSubmit() {
          return _onSubmit();
        },
        children: [/*#__PURE__*/jsxs(ModalHeader, {
          children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
            icon: faTrash
          }), "Delete"]
        }), /*#__PURE__*/jsx(ModalBody, {
          children: isFolder ? textdeleteConfirmFolder : textDeleteConfirmFile
        }), /*#__PURE__*/jsxs(ModalFooter, {
          children: [/*#__PURE__*/jsxs(Button, {
            type: "submit",
            color: "success",
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faCheck
            }), " ", labelYes]
          }), /*#__PURE__*/jsxs(Button, {
            type: "button",
            color: "danger",
            onClick: function onClick() {
              return onClose();
            },
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faTimes
            }), " ", labelCancel]
          })]
        })]
      })
    })
  });
};

var FileProgressBars = function FileProgressBars(_ref) {
  var progressStates = _ref.progressStates;
  if (progressStates.length <= 0) return null;

  var _Translations$getTran = Translations.getTranslations(),
      labelName = _Translations$getTran.labelName,
      labelSize = _Translations$getTran.labelSize,
      labelProgress = _Translations$getTran.labelProgress;

  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsxs(Table, {
      children: [/*#__PURE__*/jsx("thead", {
        children: /*#__PURE__*/jsxs("tr", {
          children: [/*#__PURE__*/jsx("th", {
            children: labelName
          }), /*#__PURE__*/jsx("th", {
            children: labelSize
          }), /*#__PURE__*/jsx("th", {
            children: labelProgress
          })]
        })
      }), /*#__PURE__*/jsx("tbody", {
        children: progressStates == null ? void 0 : progressStates.map(function (x, i) {
          return /*#__PURE__*/jsxs("tr", {
            children: [/*#__PURE__*/jsx("td", {
              children: x.fileName
            }), /*#__PURE__*/jsx("td", {
              children: formatFileSize(x.fileSize)
            }), /*#__PURE__*/jsx("td", {
              children: /*#__PURE__*/jsx(Progress, {
                value: x.progress,
                animated: true,
                children: (Math.round(x.progress * 100) / 100).toFixed(2) + "%"
              })
            })]
          }, i);
        })
      })]
    })
  });
};

var AvailableSpace = function AvailableSpace(_ref) {
  var initialSpaceUsed = _ref.initialSpaceUsed,
      spaceUsed = _ref.spaceUsed,
      spaceAvailable = _ref.spaceAvailable;

  var progressColor = function () {
    if (spaceAvailable) {
      var percentage = 100 / spaceAvailable * spaceUsed;
      if (percentage >= 90) return "danger";
      if (percentage >= 75) return "warning";
    }

    return "primary";
  }();

  if (!spaceAvailable && !initialSpaceUsed) {
    return null;
  }

  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsx("div", {
      style: {
        marginTop: "10px",
        paddingRight: "20px"
      },
      children: spaceAvailable ? /*#__PURE__*/jsxs(React.Fragment, {
        children: [/*#__PURE__*/jsx(Progress, {
          value: spaceUsed,
          max: spaceAvailable,
          striped: true,
          color: progressColor
        }), formatFileSize(spaceUsed), " / ", formatFileSize(spaceAvailable)]
      }) : /*#__PURE__*/jsx(React.Fragment, {
        children: formatFileSize(spaceUsed)
      })
    })
  });
};

/* eslint-disable max-lines */

var FileTree = function FileTree(_ref) {
  var onFileUpload = _ref.onFileUpload,
      onAddFolder = _ref.onAddFolder,
      onEditFolder = _ref.onEditFolder,
      onDeleteFolder = _ref.onDeleteFolder,
      onListFiles = _ref.onListFiles,
      onDeleteFile = _ref.onDeleteFile,
      customTranslations = _ref.customTranslations,
      language = _ref.language,
      spaceAvailable = _ref.spaceAvailable,
      initialSpaceUsed = _ref.initialSpaceUsed,
      initialFolders = _ref.initialFolders;

  var sortFolders = function sortFolders(folders) {
    return folders.sort(function (a, b) {
      return a.name.localeCompare(b.name, undefined, {
        sensitivity: "base"
      });
    });
  };

  var _useState = useState(false),
      isAddFolderModalOpen = _useState[0],
      setIsAddFolderModalOpen = _useState[1];

  var _useState2 = useState(false),
      isDeleteConfirmModalOpen = _useState2[0],
      setIsDeleteConfirmModalOpen = _useState2[1]; // we json stringify and parse it to a get a true copy of the array so we never modify the original array...


  var _useState3 = useState(sortFolders(JSON.parse(JSON.stringify(initialFolders)))),
      folderState = _useState3[0],
      setFolderState = _useState3[1];

  var _useState4 = useState(),
      selectedFolderId = _useState4[0],
      setSelectedFolderId = _useState4[1];

  var _useState5 = useState(),
      files = _useState5[0],
      setFiles = _useState5[1];

  var _useState6 = useState([]),
      progressSates = _useState6[0],
      setProgressStates = _useState6[1];

  var _useState7 = useState(false),
      isFolderTreeLoading = _useState7[0],
      setIsFolderTreeLoading = _useState7[1];

  var _useState8 = useState(Math.min(initialSpaceUsed != null ? initialSpaceUsed : 0, spaceAvailable != null ? spaceAvailable : Number.MAX_SAFE_INTEGER)),
      spaceUsed = _useState8[0],
      setSpaceUsed = _useState8[1];

  var editFolderName = useRef("");
  var deleteActionRef = useRef();
  var editDeleteActionRef = useRef();
  var deleteIsFolder = useRef(false);
  var translationsSet = useRef(false);

  if (!translationsSet.current) {
    if (language) Translations.setTranslations(language);
    if (customTranslations) Translations.setCustomTranslations(customTranslations);
    translationsSet.current = true;
  }

  var closeFoldersWithoutSubFolders = function closeFoldersWithoutSubFolders(state) {
    state.forEach(function (x) {
      if (!state.some(function (f) {
        return f.parentId == x.id;
      })) {
        x.isOpen = false;
      }
    });
  };

  var deleteFolderFromState = function deleteFolderFromState(folderId) {
    setFolderState(function (prev) {
      var copy = [].concat(prev);
      copy.splice(copy.findIndex(function (x) {
        return x.id == folderId;
      }), 1);
      closeFoldersWithoutSubFolders(copy);
      return copy;
    }); // remove selected folder if it was deleted

    if (folderId == selectedFolderId) setSelectedFolderId(undefined);
  };

  var onToggleFolder = function onToggleFolder(folderId) {
    setFolderState(function (prev) {
      var copy = [].concat(prev);
      var folder = copy.find(function (x) {
        return x.id === folderId;
      });
      if (folder) folder.isOpen = !folder.isOpen;
      return copy;
    });
  };

  var onSelectFolderAsync = function onSelectFolderAsync(folderId) {
    try {
      setFolderState(function (prev) {
        var copy = [].concat(prev);
        var folder = copy.find(function (x) {
          return x.id === folderId;
        });

        if (folder) {
          var parentFolder = copy.find(function (x) {
            return x.id === (folder == null ? void 0 : folder.parentId);
          });
          if (parentFolder) parentFolder.isOpen = true;
          setSelectedFolderId(folderId);
          onListFiles(folderId).then(function (files) {
            setFiles(files);
          });
        }

        return copy;
      });
      return Promise.resolve();
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var _onRequestAddFolder = function onRequestAddFolder(parentFolder) {
    setIsAddFolderModalOpen(true);

    editDeleteActionRef.current = function (name) {
      try {
        var parentFolderId = parentFolder == null ? null : parentFolder.id;
        return Promise.resolve(onAddFolder(parentFolderId, name)).then(function (result) {
          if (result) {
            setIsFolderTreeLoading(true);
            setIsAddFolderModalOpen(false);
            setFolderState(function (prev) {
              var copy = [].concat(prev);
              copy.push({
                id: result,
                name: name,
                parentId: parentFolderId != null ? parentFolderId : undefined
              });
              return sortFolders(copy);
            });
            setIsFolderTreeLoading(false);
          }
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  var _onRequestDeleteFolder = function onRequestDeleteFolder(folder) {
    setIsDeleteConfirmModalOpen(true);
    deleteIsFolder.current = true;

    deleteActionRef.current = function () {
      try {
        return Promise.resolve(onDeleteFolder(folder.id)).then(function (result) {
          if (result) {
            setIsFolderTreeLoading(true);
            setIsDeleteConfirmModalOpen(false);
            deleteFolderFromState(folder.id);
            setIsFolderTreeLoading(false);
          }
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  var _onRequestDeleteFile = function onRequestDeleteFile(fileId) {
    setIsDeleteConfirmModalOpen(true);
    deleteIsFolder.current = false;

    deleteActionRef.current = function () {
      try {
        return Promise.resolve(onDeleteFile(fileId)).then(function (result) {
          var _temp2 = function () {
            if (result) {
              setIsDeleteConfirmModalOpen(false);

              var _temp3 = function () {
                if (selectedFolderId) return Promise.resolve(onSelectFolderAsync(selectedFolderId)).then(function () {});
              }();

              if (_temp3 && _temp3.then) return _temp3.then(function () {});
            }
          }();

          if (_temp2 && _temp2.then) return _temp2.then(function () {});
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  var onProgressUpdate = function onProgressUpdate(id, progress) {
    setProgressStates(function (prev) {
      var clone = [].concat(prev);
      var elementIndex = clone.findIndex(function (x) {
        return x.id === id;
      });

      if (progress === 100) {
        clone.splice(elementIndex);
      } else {
        var element = clone[elementIndex];
        if (element) element.progress = progress;
      }

      return clone;
    });
  };

  var onProgressStart = function onProgressStart(file) {
    setProgressStates(function (prev) {
      var clone = [].concat(prev);
      clone.push({
        fileName: file.file.name,
        id: file.id,
        progress: 0,
        fileSize: file.file.size
      });
      return clone;
    });
  };

  var onUpdateSpaceUsed = function onUpdateSpaceUsed(x) {
    return setSpaceUsed(spaceUsed + x);
  };

  var onRequestUploadFilesAsync = function onRequestUploadFilesAsync(folderId, acceptedFiles) {
    try {
      var filesWithId = acceptedFiles.map(function (x) {
        return {
          id: uuid_1.v4(),
          file: x
        };
      });
      filesWithId.forEach(function (x) {
        return onProgressStart(x);
      });
      return Promise.resolve(onFileUpload(folderId, filesWithId, onProgressUpdate, onUpdateSpaceUsed)).then(function () {
        var _temp4 = function () {
          if (folderId == selectedFolderId) return Promise.resolve(onSelectFolderAsync(folderId)).then(function () {});
        }();

        if (_temp4 && _temp4.then) return _temp4.then(function () {});
      }); // refresh the selected folder if files have been uploaded to it
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var _onRequestEditFolder = function onRequestEditFolder(folder) {
    editFolderName.current = folder.name;
    setIsAddFolderModalOpen(true);

    editDeleteActionRef.current = function (newName) {
      try {
        return Promise.resolve(onEditFolder(folder.id, newName)).then(function (result) {
          if (result) {
            setIsFolderTreeLoading(true);
            setIsAddFolderModalOpen(false);
            setFolderState(function (prev) {
              var copy = [].concat(prev);
              var existingFolder = copy.find(function (x) {
                return x.id === folder.id;
              });
              if (existingFolder) existingFolder.name = newName;
              return sortFolders(copy);
            });
            setIsFolderTreeLoading(false);
          }
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  return /*#__PURE__*/jsxs(React.Fragment, {
    children: [/*#__PURE__*/jsxs(Row, {
      className: "neo-folder-tree-container",
      children: [/*#__PURE__*/jsxs(Col, {
        lg: 3,
        children: [/*#__PURE__*/jsx(FolderTree, {
          isLoading: isFolderTreeLoading,
          folders: folderState,
          onToggleFolder: onToggleFolder,
          onSelectFolder: onSelectFolderAsync,
          selectedFolderId: selectedFolderId,
          onRequestAddFolder: function onRequestAddFolder(parentFolder) {
            return _onRequestAddFolder(parentFolder);
          },
          onRequestEditFolder: function onRequestEditFolder(folder) {
            return _onRequestEditFolder(folder);
          },
          onRequestDeleteFolder: function onRequestDeleteFolder(folder) {
            return _onRequestDeleteFolder(folder);
          },
          onRequestUploadFiles: onRequestUploadFilesAsync
        }), /*#__PURE__*/jsx(AvailableSpace, {
          spaceAvailable: spaceAvailable,
          initialSpaceUsed: initialSpaceUsed,
          spaceUsed: spaceUsed
        })]
      }), /*#__PURE__*/jsxs(Col, {
        lg: 9,
        children: [/*#__PURE__*/jsx(FileProgressBars, {
          progressStates: progressSates
        }), /*#__PURE__*/jsx(FolderContent, {
          sizeExceeded: spaceUsed >= (spaceAvailable != null ? spaceAvailable : Number.MAX_SAFE_INTEGER),
          onSelectFolder: onSelectFolderAsync,
          folders: folderState,
          files: files,
          selectedFolderId: selectedFolderId,
          onRequestAddFolder: function onRequestAddFolder(parentFolder) {
            return _onRequestAddFolder(parentFolder);
          },
          onRequestEditFolder: function onRequestEditFolder(folder) {
            return _onRequestEditFolder(folder);
          },
          onRequestDeleteFolder: function onRequestDeleteFolder(folder) {
            return _onRequestDeleteFolder(folder);
          },
          onRequestDeleteFile: function onRequestDeleteFile(fileId) {
            return _onRequestDeleteFile(fileId);
          },
          onRequestUploadFiles: onRequestUploadFilesAsync
        })]
      })]
    }), /*#__PURE__*/jsx(AddFolderModal, {
      isOpen: isAddFolderModalOpen,
      onClose: function onClose() {
        return setIsAddFolderModalOpen(false);
      },
      initialValues: {
        name: editFolderName.current
      },
      onSubmit: function onSubmit(values) {
        try {
          var _temp6 = function () {
            if (editDeleteActionRef.current) return Promise.resolve(editDeleteActionRef.current(values.name)).then(function () {});
          }();

          return Promise.resolve(_temp6 && _temp6.then ? _temp6.then(function () {}) : void 0);
        } catch (e) {
          return Promise.reject(e);
        }
      }
    }), /*#__PURE__*/jsx(DeleteConfirmModal, {
      isFolder: deleteIsFolder.current,
      isOpen: isDeleteConfirmModalOpen,
      onClose: function onClose() {
        return setIsDeleteConfirmModalOpen(false);
      },
      onSubmit: function onSubmit() {
        try {
          var _temp8 = function () {
            if (deleteActionRef.current) return Promise.resolve(deleteActionRef.current()).then(function () {});
          }();

          return Promise.resolve(_temp8 && _temp8.then ? _temp8.then(function () {}) : void 0);
        } catch (e) {
          return Promise.reject(e);
        }
      }
    })]
  });
};

export { FileTree, formatFileSize };
//# sourceMappingURL=index.modern.js.map
