import React, { useState, useEffect, useRef } from 'react';
import { Modal, ModalBody, ModalFooter, Button, Row, Col, ButtonGroup, Breadcrumb, BreadcrumbItem, Alert, UncontrolledDropdown, DropdownToggle, DropdownMenu, DropdownItem, Spinner, ModalHeader, Table, Progress } from 'reactstrap';
import { faPaperPlane, faTimes, faFile, faFileExcel, faFileWord, faFileImage, faFileVideo, faFileAudio, faFilePdf, faFilePowerpoint, faFileArchive, faPlus, faEdit, faTrash, faFolder, faCog, faAngleDown, faDownload, faAngleRight, faFolderOpen, faUpload, faCheck } from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { FormikWrapper, InputField } from '@neolution-ch/react-formik-ui';
import { jsx, jsxs } from 'react/jsx-runtime';
import Dropzone from 'react-dropzone';

function createCommonjsModule(fn) {
  var module = { exports: {} };
	return fn(module, module.exports), module.exports;
}

var rngBrowser = createCommonjsModule(function (module) {
// Unique ID creation requires a high quality random # generator.  In the
// browser this is a little complicated due to unknown quality of Math.random()
// and inconsistent support for the `crypto` API.  We do the best we can via
// feature-detection

// getRandomValues needs to be invoked in a context where "this" is a Crypto
// implementation. Also, find the complete implementation of crypto on IE11.
var getRandomValues = (typeof(crypto) != 'undefined' && crypto.getRandomValues && crypto.getRandomValues.bind(crypto)) ||
                      (typeof(msCrypto) != 'undefined' && typeof window.msCrypto.getRandomValues == 'function' && msCrypto.getRandomValues.bind(msCrypto));

if (getRandomValues) {
  // WHATWG crypto RNG - http://wiki.whatwg.org/wiki/Crypto
  var rnds8 = new Uint8Array(16); // eslint-disable-line no-undef

  module.exports = function whatwgRNG() {
    getRandomValues(rnds8);
    return rnds8;
  };
} else {
  // Math.random()-based (RNG)
  //
  // If all else fails, use Math.random().  It's fast, but is of unspecified
  // quality.
  var rnds = new Array(16);

  module.exports = function mathRNG() {
    for (var i = 0, r; i < 16; i++) {
      if ((i & 0x03) === 0) r = Math.random() * 0x100000000;
      rnds[i] = r >>> ((i & 0x03) << 3) & 0xff;
    }

    return rnds;
  };
}
});

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */
var byteToHex = [];
for (var i = 0; i < 256; ++i) {
  byteToHex[i] = (i + 0x100).toString(16).substr(1);
}

function bytesToUuid(buf, offset) {
  var i = offset || 0;
  var bth = byteToHex;
  // join used to fix memory issue caused by concatenation: https://bugs.chromium.org/p/v8/issues/detail?id=3175#c4
  return ([
    bth[buf[i++]], bth[buf[i++]],
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]], '-',
    bth[buf[i++]], bth[buf[i++]],
    bth[buf[i++]], bth[buf[i++]],
    bth[buf[i++]], bth[buf[i++]]
  ]).join('');
}

var bytesToUuid_1 = bytesToUuid;

// **`v1()` - Generate time-based UUID**
//
// Inspired by https://github.com/LiosK/UUID.js
// and http://docs.python.org/library/uuid.html

var _nodeId;
var _clockseq;

// Previous uuid creation time
var _lastMSecs = 0;
var _lastNSecs = 0;

// See https://github.com/uuidjs/uuid for API details
function v1(options, buf, offset) {
  var i = buf && offset || 0;
  var b = buf || [];

  options = options || {};
  var node = options.node || _nodeId;
  var clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq;

  // node and clockseq need to be initialized to random values if they're not
  // specified.  We do this lazily to minimize issues related to insufficient
  // system entropy.  See #189
  if (node == null || clockseq == null) {
    var seedBytes = rngBrowser();
    if (node == null) {
      // Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
      node = _nodeId = [
        seedBytes[0] | 0x01,
        seedBytes[1], seedBytes[2], seedBytes[3], seedBytes[4], seedBytes[5]
      ];
    }
    if (clockseq == null) {
      // Per 4.2.2, randomize (14 bit) clockseq
      clockseq = _clockseq = (seedBytes[6] << 8 | seedBytes[7]) & 0x3fff;
    }
  }

  // UUID timestamps are 100 nano-second units since the Gregorian epoch,
  // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
  // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
  // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.
  var msecs = options.msecs !== undefined ? options.msecs : new Date().getTime();

  // Per 4.2.1.2, use count of uuid's generated during the current clock
  // cycle to simulate higher resolution clock
  var nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1;

  // Time since last uuid creation (in msecs)
  var dt = (msecs - _lastMSecs) + (nsecs - _lastNSecs)/10000;

  // Per 4.2.1.2, Bump clockseq on clock regression
  if (dt < 0 && options.clockseq === undefined) {
    clockseq = clockseq + 1 & 0x3fff;
  }

  // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
  // time interval
  if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
    nsecs = 0;
  }

  // Per 4.2.1.2 Throw error if too many uuids are requested
  if (nsecs >= 10000) {
    throw new Error('uuid.v1(): Can\'t create more than 10M uuids/sec');
  }

  _lastMSecs = msecs;
  _lastNSecs = nsecs;
  _clockseq = clockseq;

  // Per 4.1.4 - Convert from unix epoch to Gregorian epoch
  msecs += 12219292800000;

  // `time_low`
  var tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
  b[i++] = tl >>> 24 & 0xff;
  b[i++] = tl >>> 16 & 0xff;
  b[i++] = tl >>> 8 & 0xff;
  b[i++] = tl & 0xff;

  // `time_mid`
  var tmh = (msecs / 0x100000000 * 10000) & 0xfffffff;
  b[i++] = tmh >>> 8 & 0xff;
  b[i++] = tmh & 0xff;

  // `time_high_and_version`
  b[i++] = tmh >>> 24 & 0xf | 0x10; // include version
  b[i++] = tmh >>> 16 & 0xff;

  // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)
  b[i++] = clockseq >>> 8 | 0x80;

  // `clock_seq_low`
  b[i++] = clockseq & 0xff;

  // `node`
  for (var n = 0; n < 6; ++n) {
    b[i + n] = node[n];
  }

  return buf ? buf : bytesToUuid_1(b);
}

var v1_1 = v1;

function v4(options, buf, offset) {
  var i = buf && offset || 0;

  if (typeof(options) == 'string') {
    buf = options === 'binary' ? new Array(16) : null;
    options = null;
  }
  options = options || {};

  var rnds = options.random || (options.rng || rngBrowser)();

  // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
  rnds[6] = (rnds[6] & 0x0f) | 0x40;
  rnds[8] = (rnds[8] & 0x3f) | 0x80;

  // Copy bytes to buffer, if provided
  if (buf) {
    for (var ii = 0; ii < 16; ++ii) {
      buf[i + ii] = rnds[ii];
    }
  }

  return buf || bytesToUuid_1(rnds);
}

var v4_1 = v4;

var uuid = v4_1;
uuid.v1 = v1_1;
uuid.v4 = v4_1;

var uuid_1 = uuid;

const defaultTranslations = {
  en: {
    errorRequired: "Input is mandatory",
    textDeleteConfirmFile: "Are you sure you want to permanently delete this file?",
    textdeleteConfirmFolder: "Are you sure you want to delete this folder? All subfolders (including their files) will also be deleted.",
    textDropZone: "Drag n drop some files here, or click to select files",
    labelName: "Name",
    labelDateModified: "Date Modified",
    labelSize: "File Size",
    labelProgress: "Progress",
    labelYes: "Yes",
    labelCancel: "Cancel",
    labelSubmit: "Send",
    labelAddFolder: "Add folder",
    labelDelete: "Delete",
    labelDownload: "Download",
    warningSizeExceeded: "The file size limit has exceeded."
  },
  it: {
    errorRequired: "L'inserimento è obbligatorio",
    textDeleteConfirmFile: "Sei sicuro di voler eliminare definitivamente questo file?",
    textdeleteConfirmFolder: "Sei sicuro di voler cancellare questa cartella? Anche tutte le sottocartelle (compresi i loro file) saranno cancellate.",
    textDropZone: "Trascina qui alcuni file, o clicca per selezionare i file",
    labelName: "Nome",
    labelDateModified: "Data modificata",
    labelSize: "Dimensione del file",
    labelProgress: "Progresso",
    labelYes: "Si",
    labelCancel: "Cancella",
    labelSubmit: "Invia",
    labelAddFolder: "aggiungi nuovo",
    labelDelete: "cancellare",
    labelDownload: "scaricare",
    warningSizeExceeded: "Il limite di dimensione del file è stato superato."
  },
  de: {
    errorRequired: "Angabe zwingend",
    textDeleteConfirmFile: "Soll der Eintrag endgültig gelöscht werden?",
    textdeleteConfirmFolder: "Soll dieser Ordner endgültig gelöscht werden? Alle Unterordner und alle zugehörigen Dateien werden gelöscht.",
    textDropZone: "Dateien per Drag & Drop hierher ziehen oder klicken, um Dateien auszuwählen",
    labelName: "Name",
    labelDateModified: "Änderungsdatum",
    labelSize: "Dateigrösse",
    labelProgress: "Fortschritt",
    labelYes: "Ja",
    labelCancel: "Abbruch",
    labelSubmit: "Absenden",
    labelAddFolder: "Ordner hinzufügen",
    labelDelete: "Löschen",
    labelDownload: "Download",
    warningSizeExceeded: "Die Dateigrössenbeschränkung wurde überschritten."
  },
  fr: {
    errorRequired: "Obligatoire",
    textDeleteConfirmFile: "Êtes-vous sûr de vouloir supprimer définitivement ce fichier?",
    textdeleteConfirmFolder: "Êtes-vous sûr de vouloir supprimer ce dossier? Tous les sous-dossiers (et les fichiers qu'ils contiennent) seront également supprimés.",
    textDropZone: "Faites glisser et déposez les fichiers ici ou cliquez pour sélectionner des fichiers",
    labelName: "Nom",
    labelDateModified: "Date modifiée",
    labelSize: "Taille du fichier",
    labelProgress: "Progrès",
    labelYes: "Oui",
    labelCancel: "Annuler",
    labelSubmit: "Soumettre",
    labelAddFolder: "Ajouter le dossier",
    labelDelete: "Supprimer",
    labelDownload: "Télécharger",
    warningSizeExceeded: "La limite de taille de fichier a été dépassée."
  }
};

class Translations {
  static getTranslations() {
    return Translations.translations;
  }

  static setTranslations(lang) {
    Translations.translations = defaultTranslations[lang];
  }

  static setCustomTranslations(translations) {
    Translations.translations = translations;
  }

}

Translations.translations = defaultTranslations.en;

const AddFolderModal = ({
  onSubmit,
  isOpen,
  onClose,
  initialValues: _initialValues = {
    name: ""
  }
}) => {
  const {
    labelName,
    errorRequired,
    labelSubmit,
    labelCancel
  } = Translations.getTranslations();
  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsx(Modal, {
      isOpen: isOpen,
      toggle: () => onClose(),
      children: /*#__PURE__*/jsxs(FormikWrapper, {
        initialValues: _initialValues,
        validate: values => {
          const errors = {};
          if (!values.name) errors.name = errorRequired;
          return errors;
        },
        onSubmit: values => onSubmit(values),
        children: [/*#__PURE__*/jsx(ModalBody, {
          children: /*#__PURE__*/jsx(InputField, {
            name: "name",
            label: labelName,
            placeholder: labelName
          })
        }), /*#__PURE__*/jsxs(ModalFooter, {
          children: [/*#__PURE__*/jsxs(Button, {
            type: "submit",
            color: "primary",
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faPaperPlane
            }), " ", labelSubmit]
          }), /*#__PURE__*/jsxs(Button, {
            type: "button",
            color: "danger",
            onClick: () => onClose(),
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faTimes
            }), " ", labelCancel]
          })]
        })]
      })
    })
  });
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function formatFileSize(bytes, si = true, dp = 1) {
  const thresh = si ? 1000 : 1024;

  if (Math.abs(bytes) < thresh) {
    return `${bytes} B`;
  }

  const units = si ? ["KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"] : ["KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"];
  let u = -1;
  const r = 10 ** dp;

  do {
    // eslint-disable-next-line no-param-reassign
    bytes /= thresh;
    u += 1;
  } while (Math.round(Math.abs(bytes) * r) / r >= thresh && u < units.length - 1);

  return `${bytes.toFixed(dp)} ${units[u]}`;
}

function formatDatetime(date, format = "dd.MM.yyyy HH:mm:ss") {
  const padStart = value => value.toString().padStart(2, "0");

  const typedDate = typeof date === "string" ? new Date(date) : date;
  return format.replace(/yyyy/g, padStart(typedDate.getFullYear())).replace(/dd/g, padStart(typedDate.getDate())).replace(/MM/g, padStart(typedDate.getMonth() + 1)).replace(/HH/g, padStart(typedDate.getHours())).replace(/mm/g, padStart(typedDate.getMinutes())).replace(/ss/g, padStart(typedDate.getSeconds()));
}

const fileTypeLut = {
  ".xls": faFileExcel,
  ".xlsx": faFileExcel,
  ".doc": faFileWord,
  ".docx": faFileWord,
  ".png": faFileImage,
  ".jpeg": faFileImage,
  ".mov": faFileVideo,
  ".mp4": faFileVideo,
  ".mp3": faFileAudio,
  ".pdf": faFilePdf,
  ".ppt": faFilePowerpoint,
  ".pptx": faFilePowerpoint,
  ".rar": faFileArchive,
  ".zip": faFileArchive
};

const FileIconFormatter = ({
  fileType
}) => {
  var _fileTypeLut$fileType;

  return /*#__PURE__*/jsx(FontAwesomeIcon, {
    className: "mr-3",
    size: "3x",
    icon: (_fileTypeLut$fileType = fileTypeLut[fileType]) != null ? _fileTypeLut$fileType : faFile
  });
};

const FolderContent = ({
  folders,
  selectedFolderId,
  onRequestAddFolder,
  onRequestDeleteFolder,
  onRequestEditFolder,
  onRequestUploadFiles,
  onSelectFolder,
  onRequestDeleteFile,
  files,
  sizeExceeded: _sizeExceeded = true
}) => {
  var _files$length;

  const {
    textDropZone,
    labelDateModified,
    labelName,
    warningSizeExceeded
  } = Translations.getTranslations();
  const [breadCrumbs, setBreadCrumbs] = useState([]);
  const selectedFolder = folders.find(folder => folder.id === selectedFolderId);

  const getBreadCrumbs = () => {
    if (!selectedFolder) return [];
    const result = [];
    let start = selectedFolder;

    while (start) {
      result.push(start);
      start = folders.find(folder => {
        var _start;

        return folder.id === ((_start = start) == null ? void 0 : _start.parentId);
      });
    }

    return result.reverse();
  };

  useEffect(() => {
    setBreadCrumbs(getBreadCrumbs());
  }, [selectedFolderId]);
  const hasFolders = folders.some(x => x.parentId === selectedFolderId);
  const hasFiles = ((_files$length = files == null ? void 0 : files.length) != null ? _files$length : 0) > 0;
  return /*#__PURE__*/jsx(React.Fragment, {
    children: selectedFolder && /*#__PURE__*/jsxs(React.Fragment, {
      children: [/*#__PURE__*/jsxs(Row, {
        children: [/*#__PURE__*/jsx(Col, {
          lg: 5,
          children: /*#__PURE__*/jsx("h2", {
            style: {
              display: "inline-block"
            },
            children: selectedFolder.name
          })
        }), /*#__PURE__*/jsx(Col, {
          className: "ml-auto",
          children: /*#__PURE__*/jsx("div", {
            className: "text-right",
            children: /*#__PURE__*/jsxs(ButtonGroup, {
              children: [/*#__PURE__*/jsx(Button, {
                color: "primary",
                onClick: () => {
                  onRequestAddFolder(selectedFolder);
                },
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faPlus
                })
              }), /*#__PURE__*/jsx(Button, {
                color: "primary",
                onClick: () => {
                  onRequestEditFolder(selectedFolder);
                },
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faEdit
                })
              }), /*#__PURE__*/jsx(Button, {
                color: "primary",
                onClick: () => {
                  onRequestDeleteFolder(selectedFolder);
                },
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faTrash
                })
              })]
            })
          })
        })]
      }), /*#__PURE__*/jsx(Row, {
        children: /*#__PURE__*/jsx(Col, {
          xxl: 12,
          children: /*#__PURE__*/jsx(Breadcrumb, {
            children: breadCrumbs.length >= 2 && /*#__PURE__*/jsx(React.Fragment, {
              children: breadCrumbs.map((x, i) => /*#__PURE__*/jsx(BreadcrumbItem, {
                active: i === breadCrumbs.length - 1,
                children: /*#__PURE__*/jsx("a", {
                  onClick: () => onSelectFolder(x.id),
                  children: x.name
                })
              }, x.id))
            })
          })
        })
      }), _sizeExceeded ? /*#__PURE__*/jsx(Alert, {
        color: "danger",
        children: warningSizeExceeded
      }) : /*#__PURE__*/jsx(Dropzone, {
        onDrop: async acceptedFiles => {
          onRequestUploadFiles(selectedFolder.id, acceptedFiles);
        },
        children: ({
          getRootProps,
          getInputProps,
          isDragActive
        }) => /*#__PURE__*/jsx(Col, {
          lg: 12,
          style: {
            padding: "20px",
            borderWidth: 2,
            borderRadius: 2,
            borderColor: isDragActive ? "blue" : "#eeeeee",
            borderStyle: "dashed",
            backgroundColor: "#fafafa",
            color: "#bdbdbd",
            outline: "none",
            transition: "border .24s ease-in-out"
          },
          children: /*#__PURE__*/jsxs("div", _extends({}, getRootProps(), {
            children: [/*#__PURE__*/jsx("input", _extends({}, getInputProps())), /*#__PURE__*/jsx("p", {
              children: textDropZone
            })]
          }))
        })
      }), (hasFolders || hasFiles) && /*#__PURE__*/jsxs("table", {
        className: "mb-0 table table-hover",
        children: [/*#__PURE__*/jsx("thead", {
          children: /*#__PURE__*/jsxs("tr", {
            children: [/*#__PURE__*/jsx("th", {
              className: "align-middle bt-0",
              children: labelName
            }), /*#__PURE__*/jsx("th", {
              className: "align-middle bt-0",
              children: labelDateModified
            }), /*#__PURE__*/jsx("th", {
              className: "align-middle bt-0 text-right",
              children: "\xA0"
            })]
          })
        }), /*#__PURE__*/jsxs("tbody", {
          children: [folders.filter(x => x.parentId === selectedFolderId).map(x => /*#__PURE__*/jsxs("tr", {
            children: [/*#__PURE__*/jsx("td", {
              className: "align-middle",
              children: /*#__PURE__*/jsxs("div", {
                className: "media",
                children: [/*#__PURE__*/jsx("div", {
                  className: "media-left media-middle",
                  children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                    icon: faFolder,
                    size: "3x",
                    className: "mr-3"
                  })
                }), /*#__PURE__*/jsx("div", {
                  className: "media-body",
                  children: /*#__PURE__*/jsx("div", {
                    className: "text-inverse",
                    children: /*#__PURE__*/jsx("a", {
                      onClick: () => onSelectFolder(x.id),
                      children: x.name
                    })
                  })
                })]
              })
            }), /*#__PURE__*/jsx("td", {
              className: "align-middle"
            }), /*#__PURE__*/jsx("td", {
              className: "align-middle text-right"
            })]
          }, x.id)), files == null ? void 0 : files.map(x => /*#__PURE__*/jsxs("tr", {
            children: [/*#__PURE__*/jsx("td", {
              className: "align-middle",
              children: /*#__PURE__*/jsxs("div", {
                className: "media",
                children: [/*#__PURE__*/jsx("div", {
                  className: "media-left media-middle",
                  children: /*#__PURE__*/jsx(FileIconFormatter, {
                    fileType: x.fileType
                  })
                }), /*#__PURE__*/jsxs("div", {
                  className: "media-body",
                  children: [/*#__PURE__*/jsx("div", {
                    className: "text-inverse",
                    children: /*#__PURE__*/jsx("a", {
                      target: "_blank",
                      rel: "noreferrer",
                      href: x.url,
                      children: x.name
                    })
                  }), /*#__PURE__*/jsx("span", {
                    children: formatFileSize(x.fileSize)
                  })]
                })]
              })
            }), /*#__PURE__*/jsx("td", {
              className: "align-middle",
              children: formatDatetime(x.dateModified)
            }), /*#__PURE__*/jsx("td", {
              className: "align-middle text-right",
              children: /*#__PURE__*/jsxs(UncontrolledDropdown, {
                children: [/*#__PURE__*/jsxs(DropdownToggle, {
                  tag: "button",
                  className: "btn btn-link",
                  children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
                    icon: faCog,
                    className: "mr-2"
                  }), /*#__PURE__*/jsx(FontAwesomeIcon, {
                    icon: faAngleDown
                  })]
                }), /*#__PURE__*/jsxs(DropdownMenu, {
                  right: true,
                  children: [/*#__PURE__*/jsx("a", {
                    target: "_blank",
                    rel: "noreferrer",
                    href: x.url,
                    children: /*#__PURE__*/jsxs(DropdownItem, {
                      children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
                        icon: faDownload,
                        className: "mr-2"
                      }), "Download"]
                    })
                  }), x.isDeleteable && /*#__PURE__*/jsxs(DropdownItem, {
                    onClick: () => onRequestDeleteFile(x.id),
                    children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: faTrash,
                      className: "mr-2"
                    }), "Delete"]
                  })]
                })]
              })
            })]
          }, x.id))]
        })]
      })]
    })
  });
};

const FolderTree = props => {
  const {
    labelAddFolder
  } = Translations.getTranslations();

  if (props.isLoading) {
    return /*#__PURE__*/jsxs(React.Fragment, {
      children: ["loading...", /*#__PURE__*/jsx("div", {
        className: "spinner-border",
        role: "status",
        children: /*#__PURE__*/jsx("span", {
          className: "sr-only",
          children: "Loading..."
        })
      }), /*#__PURE__*/jsx(Spinner, {
        color: "primary",
        type: "grow",
        children: "Loading..."
      })]
    });
  }

  return /*#__PURE__*/jsxs(React.Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: "tree",
      children: props.folders && /*#__PURE__*/jsx(FolderSubtree, _extends({}, props, {
        folders: props.folders,
        parentIdToDisplay: undefined
      }))
    }), /*#__PURE__*/jsxs("a", {
      onClick: () => props.onRequestAddFolder(null),
      children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
        icon: faPlus
      }), " ", labelAddFolder]
    })]
  });
};

const FolderSubtree = props => {
  const {
    folders,
    onToggleFolder,
    onSelectFolder,
    onRequestUploadFiles,
    onRequestAddFolder,
    onRequestEditFolder,
    onRequestDeleteFolder,
    selectedFolderId,
    parentIdToDisplay
  } = props;
  const foldersToDisplay = folders == null ? void 0 : folders.filter(x => x.parentId == parentIdToDisplay);
  return /*#__PURE__*/jsx("ul", {
    children: foldersToDisplay == null ? void 0 : foldersToDisplay.map(x => /*#__PURE__*/jsx(React.Fragment, {
      children: /*#__PURE__*/jsxs("li", {
        children: [/*#__PURE__*/jsx("span", {
          style: {
            display: "inline-block",
            width: "10px",
            minWidth: "15px"
          },
          children: folders.some(f => f.parentId == x.id) && /*#__PURE__*/jsx("a", {
            onClick: () => onToggleFolder(x.id),
            children: /*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: x.isOpen ? faAngleDown : faAngleRight
            })
          })
        }), /*#__PURE__*/jsx(Dropzone, {
          noClick: true,
          onDrop: async acceptedFiles => {
            await onRequestUploadFiles(x.id, acceptedFiles);
          },
          children: ({
            getRootProps,
            getInputProps,
            open
          }) => /*#__PURE__*/jsxs("span", {
            className: "actions-trigger",
            children: [/*#__PURE__*/jsx("a", {
              onClick: () => {
                onSelectFolder(x.id);
              },
              children: /*#__PURE__*/jsxs("span", _extends({}, getRootProps(), {
                children: [/*#__PURE__*/jsx("input", _extends({}, getInputProps())), /*#__PURE__*/jsxs("span", {
                  style: {
                    fontWeight: x.id === selectedFolderId ? "bold" : "normal"
                  },
                  children: [/*#__PURE__*/jsx("span", {
                    style: {
                      minWidth: "20px",
                      display: "inline-block"
                    },
                    children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                      icon: x.isOpen ? faFolderOpen : faFolder
                    })
                  }), x.name]
                })]
              }))
            }), /*#__PURE__*/jsxs("span", {
              className: "actions ml-2",
              children: [/*#__PURE__*/jsx("a", {
                className: "pl-1",
                onClick: open,
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faUpload
                })
              }), /*#__PURE__*/jsx("a", {
                className: "pl-1",
                onClick: () => onRequestAddFolder(x),
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faPlus
                })
              }), /*#__PURE__*/jsx("a", {
                className: "pl-1",
                onClick: () => onRequestEditFolder(x),
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faEdit
                })
              }), /*#__PURE__*/jsx("a", {
                className: "pl-1",
                onClick: () => onRequestDeleteFolder(x),
                children: /*#__PURE__*/jsx(FontAwesomeIcon, {
                  icon: faTrash
                })
              })]
            })]
          })
        }), !!x.isOpen && /*#__PURE__*/jsx(FolderSubtree, _extends({}, props, {
          parentIdToDisplay: x.id
        }))]
      })
    }, x.id))
  });
};

const DeleteConfirmModal = ({
  onSubmit,
  isOpen,
  onClose,
  isFolder
}) => {
  const {
    textDeleteConfirmFile,
    textdeleteConfirmFolder,
    labelYes,
    labelCancel
  } = Translations.getTranslations();
  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsx(Modal, {
      isOpen: isOpen,
      toggle: () => onClose(),
      children: /*#__PURE__*/jsxs(FormikWrapper, {
        initialValues: {},
        onSubmit: () => onSubmit(),
        children: [/*#__PURE__*/jsxs(ModalHeader, {
          children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
            icon: faTrash
          }), "Delete"]
        }), /*#__PURE__*/jsx(ModalBody, {
          children: isFolder ? textdeleteConfirmFolder : textDeleteConfirmFile
        }), /*#__PURE__*/jsxs(ModalFooter, {
          children: [/*#__PURE__*/jsxs(Button, {
            type: "submit",
            color: "success",
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faCheck
            }), " ", labelYes]
          }), /*#__PURE__*/jsxs(Button, {
            type: "button",
            color: "danger",
            onClick: () => onClose(),
            children: [/*#__PURE__*/jsx(FontAwesomeIcon, {
              icon: faTimes
            }), " ", labelCancel]
          })]
        })]
      })
    })
  });
};

const FileProgressBars = ({
  progressStates
}) => {
  if (progressStates.length <= 0) return null;
  const {
    labelName,
    labelSize,
    labelProgress
  } = Translations.getTranslations();
  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsxs(Table, {
      children: [/*#__PURE__*/jsx("thead", {
        children: /*#__PURE__*/jsxs("tr", {
          children: [/*#__PURE__*/jsx("th", {
            children: labelName
          }), /*#__PURE__*/jsx("th", {
            children: labelSize
          }), /*#__PURE__*/jsx("th", {
            children: labelProgress
          })]
        })
      }), /*#__PURE__*/jsx("tbody", {
        children: progressStates == null ? void 0 : progressStates.map((x, i) => /*#__PURE__*/jsxs("tr", {
          children: [/*#__PURE__*/jsx("td", {
            children: x.fileName
          }), /*#__PURE__*/jsx("td", {
            children: formatFileSize(x.fileSize)
          }), /*#__PURE__*/jsx("td", {
            children: /*#__PURE__*/jsx(Progress, {
              value: x.progress,
              animated: true,
              children: `${(Math.round(x.progress * 100) / 100).toFixed(2)}%`
            })
          })]
        }, i))
      })]
    })
  });
};

const AvailableSpace = ({
  initialSpaceUsed,
  spaceUsed,
  spaceAvailable
}) => {
  const progressColor = (() => {
    if (spaceAvailable) {
      const percentage = 100 / spaceAvailable * spaceUsed;
      if (percentage >= 90) return "danger";
      if (percentage >= 75) return "warning";
    }

    return "primary";
  })();

  if (!spaceAvailable && !initialSpaceUsed) {
    return null;
  }

  return /*#__PURE__*/jsx(React.Fragment, {
    children: /*#__PURE__*/jsx("div", {
      style: {
        marginTop: "10px",
        paddingRight: "20px"
      },
      children: spaceAvailable ? /*#__PURE__*/jsxs(React.Fragment, {
        children: [/*#__PURE__*/jsx(Progress, {
          value: spaceUsed,
          max: spaceAvailable,
          striped: true,
          color: progressColor
        }), formatFileSize(spaceUsed), " / ", formatFileSize(spaceAvailable)]
      }) : /*#__PURE__*/jsx(React.Fragment, {
        children: formatFileSize(spaceUsed)
      })
    })
  });
};

/* eslint-disable max-lines */

const FileTree = ({
  onFileUpload,
  onAddFolder,
  onEditFolder,
  onDeleteFolder,
  onListFiles,
  onDeleteFile,
  customTranslations,
  language,
  spaceAvailable,
  initialSpaceUsed,
  initialFolders
}) => {
  const sortFolders = folders => folders.sort((a, b) => a.name.localeCompare(b.name, undefined, {
    sensitivity: "base"
  }));

  const [isAddFolderModalOpen, setIsAddFolderModalOpen] = useState(false);
  const [isDeleteConfirmModalOpen, setIsDeleteConfirmModalOpen] = useState(false); // we json stringify and parse it to a get a true copy of the array so we never modify the original array...

  const [folderState, setFolderState] = useState(sortFolders(JSON.parse(JSON.stringify(initialFolders))));
  const [selectedFolderId, setSelectedFolderId] = useState();
  const [files, setFiles] = useState();
  const [progressSates, setProgressStates] = useState([]);
  const [isFolderTreeLoading, setIsFolderTreeLoading] = useState(false);
  const [spaceUsed, setSpaceUsed] = useState(Math.min(initialSpaceUsed != null ? initialSpaceUsed : 0, spaceAvailable != null ? spaceAvailable : Number.MAX_SAFE_INTEGER));
  const editFolderName = useRef("");
  const deleteActionRef = useRef();
  const editDeleteActionRef = useRef();
  const deleteIsFolder = useRef(false);
  const translationsSet = useRef(false);

  if (!translationsSet.current) {
    if (language) Translations.setTranslations(language);
    if (customTranslations) Translations.setCustomTranslations(customTranslations);
    translationsSet.current = true;
  }

  const closeFoldersWithoutSubFolders = state => {
    state.forEach(x => {
      if (!state.some(f => f.parentId == x.id)) {
        x.isOpen = false;
      }
    });
  };

  const deleteFolderFromState = folderId => {
    setFolderState(prev => {
      const copy = [...prev];
      copy.splice(copy.findIndex(x => x.id == folderId), 1);
      closeFoldersWithoutSubFolders(copy);
      return copy;
    }); // remove selected folder if it was deleted

    if (folderId == selectedFolderId) setSelectedFolderId(undefined);
  };

  const onToggleFolder = folderId => {
    setFolderState(prev => {
      const copy = [...prev];
      var folder = copy.find(x => x.id === folderId);
      if (folder) folder.isOpen = !folder.isOpen;
      return copy;
    });
  };

  const onSelectFolderAsync = async folderId => {
    setFolderState(prev => {
      const copy = [...prev];
      var folder = copy.find(x => x.id === folderId);

      if (folder) {
        var parentFolder = copy.find(x => x.id === (folder == null ? void 0 : folder.parentId));
        if (parentFolder) parentFolder.isOpen = true;
        setSelectedFolderId(folderId);
        onListFiles(folderId).then(files => {
          setFiles(files);
        });
      }

      return copy;
    });
  };

  const onRequestAddFolder = parentFolder => {
    setIsAddFolderModalOpen(true);

    editDeleteActionRef.current = async name => {
      const parentFolderId = parentFolder == null ? null : parentFolder.id;
      const result = await onAddFolder(parentFolderId, name);

      if (result) {
        setIsFolderTreeLoading(true);
        setIsAddFolderModalOpen(false);
        setFolderState(prev => {
          const copy = [...prev];
          copy.push({
            id: result,
            name: name,
            parentId: parentFolderId != null ? parentFolderId : undefined
          });
          return sortFolders(copy);
        });
        setIsFolderTreeLoading(false);
      }
    };
  };

  const onRequestDeleteFolder = folder => {
    setIsDeleteConfirmModalOpen(true);
    deleteIsFolder.current = true;

    deleteActionRef.current = async () => {
      const result = await onDeleteFolder(folder.id);

      if (result) {
        setIsFolderTreeLoading(true);
        setIsDeleteConfirmModalOpen(false);
        deleteFolderFromState(folder.id);
        setIsFolderTreeLoading(false);
      }
    };
  };

  const onRequestDeleteFile = fileId => {
    setIsDeleteConfirmModalOpen(true);
    deleteIsFolder.current = false;

    deleteActionRef.current = async () => {
      const result = await onDeleteFile(fileId);

      if (result) {
        setIsDeleteConfirmModalOpen(false);
        if (selectedFolderId) await onSelectFolderAsync(selectedFolderId);
      }
    };
  };

  const onProgressUpdate = (id, progress) => {
    setProgressStates(prev => {
      const clone = [...prev];
      const elementIndex = clone.findIndex(x => x.id === id);

      if (progress === 100) {
        clone.splice(elementIndex);
      } else {
        var element = clone[elementIndex];
        if (element) element.progress = progress;
      }

      return clone;
    });
  };

  const onProgressStart = file => {
    setProgressStates(prev => {
      const clone = [...prev];
      clone.push({
        fileName: file.file.name,
        id: file.id,
        progress: 0,
        fileSize: file.file.size
      });
      return clone;
    });
  };

  const onUpdateSpaceUsed = x => setSpaceUsed(spaceUsed + x);

  const onRequestUploadFilesAsync = async (folderId, acceptedFiles) => {
    const filesWithId = acceptedFiles.map(x => ({
      id: uuid_1.v4(),
      file: x
    }));
    filesWithId.forEach(x => onProgressStart(x));
    await onFileUpload(folderId, filesWithId, onProgressUpdate, onUpdateSpaceUsed); // refresh the selected folder if files have been uploaded to it

    if (folderId == selectedFolderId) await onSelectFolderAsync(folderId);
  };

  const onRequestEditFolder = folder => {
    editFolderName.current = folder.name;
    setIsAddFolderModalOpen(true);

    editDeleteActionRef.current = async newName => {
      const result = await onEditFolder(folder.id, newName);

      if (result) {
        setIsFolderTreeLoading(true);
        setIsAddFolderModalOpen(false);
        setFolderState(prev => {
          const copy = [...prev];
          const existingFolder = copy.find(x => x.id === folder.id);
          if (existingFolder) existingFolder.name = newName;
          return sortFolders(copy);
        });
        setIsFolderTreeLoading(false);
      }
    };
  };

  return /*#__PURE__*/jsxs(React.Fragment, {
    children: [/*#__PURE__*/jsxs(Row, {
      className: "neo-folder-tree-container",
      children: [/*#__PURE__*/jsxs(Col, {
        lg: 3,
        children: [/*#__PURE__*/jsx(FolderTree, {
          isLoading: isFolderTreeLoading,
          folders: folderState,
          onToggleFolder: onToggleFolder,
          onSelectFolder: onSelectFolderAsync,
          selectedFolderId: selectedFolderId,
          onRequestAddFolder: parentFolder => onRequestAddFolder(parentFolder),
          onRequestEditFolder: folder => onRequestEditFolder(folder),
          onRequestDeleteFolder: folder => onRequestDeleteFolder(folder),
          onRequestUploadFiles: onRequestUploadFilesAsync
        }), /*#__PURE__*/jsx(AvailableSpace, {
          spaceAvailable: spaceAvailable,
          initialSpaceUsed: initialSpaceUsed,
          spaceUsed: spaceUsed
        })]
      }), /*#__PURE__*/jsxs(Col, {
        lg: 9,
        children: [/*#__PURE__*/jsx(FileProgressBars, {
          progressStates: progressSates
        }), /*#__PURE__*/jsx(FolderContent, {
          sizeExceeded: spaceUsed >= (spaceAvailable != null ? spaceAvailable : Number.MAX_SAFE_INTEGER),
          onSelectFolder: onSelectFolderAsync,
          folders: folderState,
          files: files,
          selectedFolderId: selectedFolderId,
          onRequestAddFolder: parentFolder => onRequestAddFolder(parentFolder),
          onRequestEditFolder: folder => onRequestEditFolder(folder),
          onRequestDeleteFolder: folder => onRequestDeleteFolder(folder),
          onRequestDeleteFile: fileId => onRequestDeleteFile(fileId),
          onRequestUploadFiles: onRequestUploadFilesAsync
        })]
      })]
    }), /*#__PURE__*/jsx(AddFolderModal, {
      isOpen: isAddFolderModalOpen,
      onClose: () => setIsAddFolderModalOpen(false),
      initialValues: {
        name: editFolderName.current
      },
      onSubmit: async values => {
        if (editDeleteActionRef.current) await editDeleteActionRef.current(values.name);
      }
    }), /*#__PURE__*/jsx(DeleteConfirmModal, {
      isFolder: deleteIsFolder.current,
      isOpen: isDeleteConfirmModalOpen,
      onClose: () => setIsDeleteConfirmModalOpen(false),
      onSubmit: async () => {
        if (deleteActionRef.current) await deleteActionRef.current();
      }
    })]
  });
};

export { FileTree, formatFileSize };
//# sourceMappingURL=index.modern.mjs.map
