import React from "react";
import { FileItem, FileWithId, SupportedLangs, Folder } from "./types";
import { ITranslations } from "../Utils/translations";
import "./css/style.css";
export declare type OnAddFolderFn = (parentId: string | null, name: string) => Promise<string>;
export declare type OnEditFolderFn = (folderId: string, name: string) => Promise<boolean>;
export declare type OnDeleteFolderFn = (folderId: string) => Promise<boolean>;
export declare type OnListFilesFn = (folderId: string) => Promise<FileItem[]>;
export declare type OnDeleteFileFn = (fileId: string) => Promise<boolean>;
export declare type OnFileUploadFn = (folderId: string, acceptedFiles: FileWithId[], onProgressUpdate: (id: string, progress: number) => void, onUpdateSpaceUsed: (x: number) => void) => Promise<boolean>;
interface FileTreeProps {
    /** Function called when files are uploaded (via drop or click) */
    onFileUpload: OnFileUploadFn;
    /** Function called when a new folder is added */
    onAddFolder: OnAddFolderFn;
    /** Function called when a folder is edited */
    onEditFolder: OnEditFolderFn;
    /** Function called when a folder is deleted */
    onDeleteFolder: OnDeleteFolderFn;
    /** Function called when files are listed */
    onListFiles: OnListFilesFn;
    /** Function called when we need to delete a file */
    onDeleteFile: OnDeleteFileFn;
    /** The language to be used for the labels. If your lang is not supported you can use the customTranslations prop. */
    language?: SupportedLangs;
    /** Custom translations (defaults to english ones) */
    customTranslations?: ITranslations;
    /** The initial space used in bytes. Will be updated when files are uploaded (via the onUpdateSpaceUsed callback) */
    initialSpaceUsed?: number;
    /** The space available in bytes */
    spaceAvailable?: number;
    /** The initial folder structure */
    initialFolders: Folder[];
}
/**
 * File Tree component. Renders a file tree with clickable actions.
 */
declare const FileTree: React.FC<FileTreeProps>;
export { FileTree, FileTreeProps, FileItem, Folder };
